#include <cstdlib>
#include <iostream>
#include <vector>
#include <string>
#include <fstream>
#include <sstream>
#include <stdio.h>

/*
Smooth - A helper program for BioVEC.
Makes weighted averages of LAMMPS dumpfiles, for use with BioVEC.


Released under GPL.

Erik Abrahamsson, 2009
erik@phas.ubc.ca

*/

using namespace std;

int main(int argc, char *argv[])
{
   
   vector<float> X, Y, Z;
   vector<float> Q0, Q1, Q2, Q3;
   vector<float> avgX, avgY, avgZ;
   //vector<float> avgQ0, avgQ1, avgQ2, avgQ3;
   
   vector<int> atom;
   
   int ntime = 0;
   int ios = 0;
   int natoms, step, skip;
   int k;
   
   string inname, outname, text, cstring;
   char junk[80];
   
   ifstream infile;
   ofstream outfile;
   
   if (argc > 1) {
      inname = argv[1];
      if (argc > 2) {
         skip = atoi(argv[2]);
         if (skip <= 0) {
            skip = 1;
         }
      }
      else {
         skip = 1;
      }
   }
   else {
      cout << "Please enter name of original dump file: " ;
      getline (cin, inname);
      cout << "Please enter skip length: " ;
      cin >> skip;
      if (skip <= 0) {
         skip = 10;
      }
   }
    //if (argc > 2) {
//       step = atoi(argv[2]);
//       if (step <= 0) {
//          step = 10;
//       }
//    }
//    else {
//       cout << "Please enter step length: " ;
//       cin >> step;
//       if (step <= 0) {
//          step = 10;
//       }
//    }
   step = 10;
   
   //if (step <= 0) step = 10;
   cout << "Step: " << step << endl;
   cout << "Skip: " << skip << endl;
   
   infile.open(inname.c_str());
   if (infile.is_open()) {
      cout << "Infile " << inname << " opened" << endl;
      
      cout << "Reading..." << endl;
      
      while (cstring != "ITEM: NUMBER OF ATOMS" && !infile.eof()) {
         getline(infile, text);
         cstring.assign(text, 0, 21);
      }
      getline (infile, text);
      istringstream(text) >> natoms;
      
      cout << "Number of atoms: " << natoms << endl;

      // rewind and read complete file
      infile.seekg(0, ios::beg);
      cstring.clear();  
      
      while (!infile.eof()) {
         while (cstring != "ITEM: ATOMS" && !infile.eof()) {
            getline(infile, text);
            cstring.assign(text, 0, 11);
         }
         cstring.clear();
         for (int i = 1; i < natoms; i++) {
            getline (infile, text);
         }
         ntime++;
      }
      if (text == "") {
         ntime--; // One line too much, due to blank line at end of file   
      }
      
      if (step > ntime-1) {
         cout << "The step is too large. Only " << ntime 
              << "time steps in then file."  << endl;
         cout << "Press Enter to exit.";
              cin.clear();
         cin.ignore(1,0);
         std::exit(0);
      }
      infile.clear();
      infile.seekg(0, ios::beg); // Rewind file, again
      
      cout << "Number of time steps: " << ntime << endl;
      
      outname = "avg";
      k = sprintf(junk, "-%d-%d-", step, skip);
      outname +=  junk;
      outname += inname; 
      
      cout << "The new data will be in the file " << outname << endl;
      
      outfile.open(outname.c_str());
      if(outfile.is_open()) {
         X.resize(ntime*natoms);
         Y.resize(ntime*natoms);
         Z.resize(ntime*natoms);
         avgX.resize(ntime*natoms);
         avgY.resize(ntime*natoms);
         avgZ.resize(ntime*natoms);
         Q0.resize(ntime*natoms);
         Q1.resize(ntime*natoms);
         Q2.resize(ntime*natoms);
         Q3.resize(ntime*natoms);
         atom.resize(natoms);
         
         cout << "Matrices allocated " << endl;
         cout << "Reading data" << endl;
         
         for (int t = 0; t < ntime; t++) {
            // Read headers 
            while (cstring != "ITEM: ATOMS" && !infile.eof()) {
               getline(infile, text);
               cstring.assign(text, 0, 11);
            }
            cstring.clear();
            
            for (int i = 0; i < natoms; i++) {
               getline (infile, text);
               istringstream(text) >> ios >> atom.at(i) >> 
                  X.at(t*natoms+i) >> Y.at(t*natoms+i) >> Z.at(t*natoms+i) >> 
                  Q0.at(t*natoms+i) >> Q1.at(t*natoms+i) >> 
                  Q2.at(t*natoms+i) >>  Q3.at(t*natoms+i);
                  avgX.at(t*natoms+i) = 0.0;
                  avgY.at(t*natoms+i) = 0.0;
                  avgZ.at(t*natoms+i) = 0.0;
            }
         }
         
         cout << "Done reading data" << endl;
         cout << "Calculating averages" << endl;
         
         for (int t = step/2; t < ntime-step/2; t = t + skip) {
            // Write headers for each time step
            outfile << "ITEM: TIMESTEP" << endl;
            outfile << t-step/2 << endl;
            outfile << "ITEM: NUMBER OF ATOMS" << endl;
            outfile << natoms << endl;
            //outfile << "ITEM: BOX BOUNDS" << endl;
            //outfile << "-1500 1500" << endl;
            //outfile << "-1500 1500" << endl;
            //outfile << "-1500 1500" << endl;
            outfile << "ITEM: ATOMS" << endl;
            
            for (int i = 0; i < natoms; i++) {
               for (int j = -step/2; j < step/2+1; j++) {
                  //cout << t << ", " << (t+j)*natoms+i << " " << i << ", " 
                  //     << j << endl;
                  avgX.at(t*natoms+i) = 
                     avgX.at(t*natoms+i) + 
                     X.at((t+j)*natoms+i)*1.0/((step+2.0)/3.0*(abs(j)+1.0));
                  avgY.at(t*natoms+i) = 
                     avgY.at(t*natoms+i) + 
                     Y.at((t+j)*natoms+i)*1.0/((step+2.0)/3.0*(abs(j)+1.0));
                  avgZ.at(t*natoms+i) = 
                     avgZ.at(t*natoms+i) + 
                     Z.at((t+j)*natoms+i)*1.0/((step+2.0)/3.0*(abs(j)+1.0));
               }
               outfile << i+1 << " " << atom.at(i) << " " << 
                  avgX.at(t*natoms+i) << " " << 
                  avgY.at(t*natoms+i) << " " << 
                  avgZ.at(t*natoms+i) << " " <<
                  Q0.at(t*natoms+i) << " " <<  
                  Q1.at(t*natoms+i) << " " << 
                  Q2.at(t*natoms+i) << " " << 
                  Q3.at(t*natoms+i) << endl;
            }
         }
      }
      else {
         cout << outname << ": The file could not be opened. Something's " <<
         "wrong" << endl;
      }
   }
   else {
      cout << inname << ": No such file. Try again." << endl;
   }
   
   cout << "All done." << endl;
   cout << "Bye." << endl;
   //system("PAUSE");
   return EXIT_SUCCESS;
}
